@@ -301,6 +301,7 @@ int AsciiFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 			if(vargs[2].number().getBoolean() && str.length() > 1) {
 				mstruct.clearVector();
 				mstruct.resizeVector(str.length(), m_zero);
+				if(mstruct.size() < str.length()) return 0;
 				for(size_t i = 0; i < str.length(); i++) {
 					mstruct[i] = (long int) ((unsigned char) str[i]);
 				}
@@ -421,7 +422,7 @@ int ConcatenateFunction::calculate(MathStructure &mstruct, const MathStructure &
 	for(size_t i = 0; i < vargs.size(); i++) {
 		str += vargs[i].symbol();
 	}
-	mstruct = str;
+	mstruct.set(str, false, true);
 	return 1;
 }
 LengthFunction::LengthFunction() : MathFunction("len", 1) {
@@ -472,7 +473,7 @@ int ReplaceFunction::calculate(MathStructure &mstruct, const MathStructure &varg
 		MathStructure meval(vargs[2]);
 		CALCULATOR->beginTemporaryStopMessages();
 		meval.eval(eo);
-		if(vargs[1].isVector() && meval.isVector() && vargs[1].size() == meval.size()) {
+		if(meval.isVector() && vargs[1].size() == meval.size()) {
 			CALCULATOR->endTemporaryStopMessages(true);
 			for(size_t i = 0; i < vargs[1].size(); i++) {
 				if(vargs[1][i].isFunction() && meval[i].isFunction() && vargs[1][i].size() == 0 && meval[i].size() == 0) {
@@ -1003,10 +1004,11 @@ int PlotFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 	bool use_step_size = vargs[5].number().getBoolean() || !vargs[3].isInteger() || vargs[3].number() < 2;
 	mstruct = vargs[0];
 	CALCULATOR->beginTemporaryStopIntervalArithmetic();
-	if(!mstruct.contains(vargs[4], true) || (!mstruct.isVector() && !mstruct.representsScalar())) {
+	if(!mstruct.contains(vargs[4], true) || (!mstruct.isVector() && (!mstruct.isFunction() || (mstruct.function()->id() != FUNCTION_ID_HORZCAT && mstruct.function()->id() != FUNCTION_ID_VERTCAT)) && !mstruct.representsScalar())) {
 		CALCULATOR->beginTemporaryStopMessages();
 		mstruct.eval(eo2);
 		CALCULATOR->endTemporaryStopMessages();
+		if(mstruct.isFunction() && (mstruct.function()->id() == FUNCTION_ID_HORZCAT || mstruct.function()->id() == FUNCTION_ID_VERTCAT)) mstruct.setType(STRUCT_VECTOR);
 		if(!mstruct.isVector() && vargs[0].contains(vargs[4], true)) mstruct = vargs[0];
 	} else {
 		eo2.calculate_functions = false;
@@ -1018,6 +1020,8 @@ int PlotFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		eo2.calculate_functions = eo.calculate_functions;
 		eo2.expand = eo.expand;
 	}
+	if(mstruct.isFunction() && (mstruct.function()->id() == FUNCTION_ID_HORZCAT || mstruct.function()->id() == FUNCTION_ID_VERTCAT)) mstruct.setType(STRUCT_VECTOR);
+	if(mstruct.isMatrix() && mstruct.columns() == 1 && mstruct.rows() > 1) mstruct.transposeMatrix();
 	CALCULATOR->endTemporaryStopIntervalArithmetic();
 	vector<MathStructure> x_vectors, y_vectors;
 	vector<PlotDataParameters*> dpds;
